/*
 * JBoss, Home of Professional Open Source
 * Copyright 2014, Red Hat, Inc. and/or its affiliates, and individual
 * contributors by the @authors tag. See the copyright.txt in the
 * distribution for a full listing of individual contributors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jboss.as.quickstarts.wshelloworld;

import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.logging.Logger;
import javax.inject.Inject;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jboss.as.quickstarts.wshelloworld.Client;
import org.jboss.as.quickstarts.wshelloworld.HelloWorldService;

@WebServlet("/client")
public class WSClientServlet extends HttpServlet {

    private static final long serialVersionUID = 127759768859064589L;

    private static final Logger LOGGER = Logger.getLogger(WSClientServlet.class.getName());

    /** 
     * The name of the WAR Archive that will be used by Arquillian to deploy the application.
     */
    private static final String APP_NAME = "jboss-helloworld-ws";
    /** 
     * The path of the WSDL endpoint in relation to the deployed web application.
     */
    private static final String WSDL_PATH = "HelloWorldService?wsdl";
    
    /** 
     * The name for the Server URL System Property.
     */
    private static final String SERVER_URL_PROPERTY = "serverUrl";
    /** 
     * The Default Server URL if one isn't specified as a System Property
     */
    private static final String DEFAULT_SERVER_URL = "http://localhost:8080/";

    private HelloWorldService client;

    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

        LOGGER.info(">>> WSClientServlet#doGet()");

        response.setContentType("text/html");
        PrintWriter out = response.getWriter();

        String deploymentUrl = System.getProperty(SERVER_URL_PROPERTY);
    
        try {
            // Check that the server URL property was set. If it wasn't then use the default.
            if (deploymentUrl == null || deploymentUrl.isEmpty()) {
                deploymentUrl = DEFAULT_SERVER_URL;
            }
        
            // Ensure that the URL ends with a forward slash
            if (!deploymentUrl.endsWith("/")) {
                deploymentUrl += "/";
            }
        
            // Ensure the App Name is specified in the URL
            if (!deploymentUrl.matches(".*" + APP_NAME + ".*"))
            {
                deploymentUrl += APP_NAME + "/";
            }
        
            // Add the WDSL Document location to the URL
            deploymentUrl += WSDL_PATH;
        
            LOGGER.info("WSDL Deployment URL: " + deploymentUrl);

            // client = new Client(new URL(deploymentUrl, WSDL_PATH));
            client = new Client(new URL(deploymentUrl));

        } catch (MalformedURLException e) {
            e.printStackTrace();
        }

        {
            String result = testHello();
            out.println("testHello() = " + result);
            out.println("<br>");
        }
        {
            String result = testHelloName("John");
            out.println("testHelloName() = " + result);
            out.println("<br>");
        }
        {
            String result = testHelloNames();
            out.println("testHelloName() = " + result);
            out.println("<br>");
        }

        LOGGER.info("<<< WSClientServlet#doGet()");

    }

    String testHello() {
        LOGGER.info("[Client] Requesting the WebService to say Hello.");

        // Get a response from the WebService
        String response = client.sayHello();

        LOGGER.info("[WebService] " + response);

        return response;
    }

    String testHelloName(String name) {
        LOGGER.info("[Client] Requesting the WebService to say Hello to " + name);

        // Get a response from the WebService
        String response = client.sayHelloToName(name);

        LOGGER.info("[WebService] " + response);

        return response;
    }

    String testHelloNames() {
        LOGGER.info("[Client] Requesting the WebService to say Hello to John, Mary and Mark.");

        // Create the array of names for the WebService to say hello to.
        final List<String> names = new ArrayList<String>();
        names.add("John");
        names.add("Mary");
        names.add("Mark");

        // Get a response from the WebService
        final String response = client.sayHelloToNames(names);

        LOGGER.info("[WebService] " + response);

        return response;
    }

}
